@@ -1,6 +1,6 @@
 from pubnub import utils
 from pubnub.endpoints.endpoint import Endpoint
-from pubnub.errors import PNERR_RESOURCES_MISSING, PNERR_TTL_MISSING, PNERR_INVALID_META
+from pubnub.errors import PNERR_TTL_MISSING, PNERR_INVALID_META, PNERR_RESOURCES_MISSING
 from pubnub.exceptions import PubNubException
 from pubnub.enums import HttpMethod, PNOperationType
 from pubnub.models.consumer.v3.access_manager import PNGrantTokenResult
@@ -9,20 +9,14 @@ from pubnub.models.consumer.v3.access_manager import PNGrantTokenResult
 class GrantToken(Endpoint):
     GRANT_TOKEN_PATH = "/v3/pam/%s/grant"
 
-    READ = 1
-    WRITE = 2
-    MANAGE = 4
-    DELETE = 8
-    CREATE = 16
-
     def __init__(self, pubnub):
         Endpoint.__init__(self, pubnub)
         self._ttl = None
         self._meta = None
+        self._authorized_uuid = None
         self._channels = []
         self._groups = []
-        self._users = []
-        self._spaces = []
+        self._uuids = []
 
         self._sort_params = True
 
@@ -34,6 +28,10 @@ class GrantToken(Endpoint):
         self._meta = meta
         return self
 
+    def authorized_uuid(self, uuid):
+        self._authorized_uuid = uuid
+        return self
+
     def channels(self, channels):
         self._channels = channels
         return self
@@ -42,19 +40,15 @@ class GrantToken(Endpoint):
         self._groups = groups
         return self
 
-    def users(self, users):
-        self._users = users
-        return self
-
-    def spaces(self, spaces):
-        self._spaces = spaces
+    def uuids(self, uuids):
+        self._uuids = uuids
         return self
 
     def custom_params(self):
         return {}
 
     def build_data(self):
-        params = {'ttl': str(self._ttl)}
+        params = {'ttl': int(self._ttl)}
 
         permissions = {}
         resources = {}
@@ -62,13 +56,14 @@ class GrantToken(Endpoint):
 
         utils.parse_resources(self._channels, "channels", resources, patterns)
         utils.parse_resources(self._groups, "groups", resources, patterns)
-        utils.parse_resources(self._users, "users", resources, patterns)
-        utils.parse_resources(self._spaces, "spaces", resources, patterns)
+        utils.parse_resources(self._uuids, "uuids", resources, patterns)
+        utils.parse_resources(self._uuids, "users", resources, patterns)
+        utils.parse_resources(self._uuids, "spaces", resources, patterns)
 
         permissions['resources'] = resources
         permissions['patterns'] = patterns
 
-        if self._meta is not None:
+        if self._meta:
             if isinstance(self._meta, dict):
                 permissions['meta'] = self._meta
             else:
@@ -76,6 +71,9 @@ class GrantToken(Endpoint):
         else:
             permissions['meta'] = {}
 
+        if self._authorized_uuid:
+            permissions["uuid"] = self._authorized_uuid
+
         params['permissions'] = permissions
 
         return utils.write_value_as_string(params)
@@ -111,12 +109,9 @@ class GrantToken(Endpoint):
         return "Grant Token"
 
     def validate_resources(self):
-        if (self._channels is None or len(self._channels) == 0) and \
-           (self._groups is None or len(self._groups) == 0) and \
-           (self._users is None or len(self._users) == 0) and \
-           (self._spaces is None or len(self._spaces) == 0):
+        if not any((self._channels, self._groups, self._uuids)):
             raise PubNubException(pn_error=PNERR_RESOURCES_MISSING)
 
     def validate_ttl(self):
-        if self._ttl is None:
+        if not self._ttl:
             raise PubNubException(pn_error=PNERR_TTL_MISSING)
